import { HttpClient, HttpParams } from '@angular/common/http';
import { ɵɵdefineInjectable, ɵɵinject, Injectable, APP_INITIALIZER, isDevMode } from '@angular/core';
import { throwError, of, Subject } from 'rxjs';
import { SysType, QueryResult, EditResult } from '@yukawa/chain-base-angular-domain';
import { map, mergeMap, concatAll, concatMap, toArray, delay, startWith, switchMap, shareReplay, tap } from 'rxjs/operators';

// import '@yukawa/chain-base-angular-domain/chaos-sys-core';
// declare const SysType;
const TAG = 'ConfigService';
class ConfigService {
    constructor(http) {
        this.http = http;
        /**
         * @deprecated
         */
        this.configUrl = "config.json";
        this.config = {};
    }
    /**
     * @deprecated
     */
    initConfig() {
        return new Promise((resolve, reject) => {
            console.warn(TAG, "loadConfig is deprecated use initConfigs");
            this.loadConfig()
                .subscribe((data) => {
                this.config = data;
                resolve(true);
            }, (error) => {
                console.error(TAG, `initConfig  ${error}`);
                //reject(error)
                return throwError(error || 'Server error');
            });
        });
    }
    initConfigs(locations = ['config-base.json', 'config.json'], rejectNotFound = false) {
        return new Promise((resolve, reject) => {
            for (let idx = 0; idx < locations.length; idx++) {
                let loc = locations[idx];
                this.http.get(loc).subscribe(cfg => {
                    this.config = SysType.objectMix(this.config || {}, cfg);
                    //this.config = this.deepCopy(this.config||{}, cfg);
                    if (idx == locations.length - 1) {
                        resolve(this.config);
                    }
                }, error => {
                    if (rejectNotFound) {
                        reject(error);
                        return;
                    }
                    if (idx == locations.length - 1) {
                        resolve(this.config);
                    }
                });
            }
        });
    }
    getBaseUrl(id = "") {
        return id == "" ? this.config.baseUrl : this.config[id];
    }
    getValue(id) {
        return this.config[id];
    }
    formatUrl(id = "baseUrl") {
        let cfg = this.config;
        if (!cfg)
            return cfg;
        cfg = cfg[id];
        if (cfg.startsWith('/') || cfg.startsWith(':'))
            cfg = this.config.baseUrl + cfg;
        return cfg;
    }
    /**
     * @deprecated
     */
    getConfig(id) {
        console.warn(TAG, "getConfig is deprecated use initConfigs");
        if (!id)
            return this.config;
        return `${this.getBaseUrl()}${this.config[id]}`;
    }
    /**
     * @deprecated
     */
    loadConfig(url = 'config.json') {
        console.warn(TAG, "loadConfig is deprecated use initConfigs " + url);
        return this.http.get(url);
    }
}
/** @nocollapse */ ConfigService.ɵprov = ɵɵdefineInjectable({ factory: function ConfigService_Factory() { return new ConfigService(ɵɵinject(HttpClient)); }, token: ConfigService, providedIn: "root" });
ConfigService.decorators = [
    { type: Injectable, args: [{
                providedIn: 'root'
            },] }
];
/** @nocollapse */
ConfigService.ctorParameters = () => [
    { type: HttpClient }
];
function ConfigFactory(config) {
    return () => config.initConfigs();
}
function init() {
    return {
        provide: APP_INITIALIZER,
        useFactory: ConfigFactory,
        deps: [ConfigService],
        multi: true
    };
}
const ConfigModule = {
    init: init
};

// @Injectable({
//   providedIn: 'root'
// })
class UrlConfigService {
    constructor(cfg, configService) {
        this.cfg = cfg;
        this.configService = configService;
        console.log("xxxxxxxxxxxxx");
    }
    getBackendUrl() {
        return this.configService.formatUrl(this.cfg);
    }
}

const TAG$1 = 'RestAspect';
class RestAspect {
    constructor(http, configService, serviceUrl) {
        this.http = http;
        this.configService = configService;
        this.serviceUrl = serviceUrl;
    }
    formatBaseUrl(path = '') {
        return this.configService.getBaseUrl('baseUrl') + path;
    }
    formatServiceUrl(path = '') {
        return this.serviceUrl + path + '/';
    }
    // noinspection JSMethodCanBeStatic
    get verbose() {
        return isDevMode();
    }
    search(url, filter, method = 'post') {
        let m = method || 'post';
        if (this.verbose)
            console.debug(TAG$1, "search:", m, url, filter);
        return this.http[m](url, filter).pipe(map((r) => {
            let result = {
                took: r.took,
                timedOut: r.timed_out,
                rows: []
            };
            let h = r.hits;
            if (h) {
                result.total = h.total;
                for (let idx = 0; idx < h.hits.length; idx++) {
                    const hit = h.hits[idx];
                    let source = hit._source;
                    // console.log('DeviceService', hit);
                    result.rows.push(source);
                }
            }
            return result;
        }));
    }
    query(url, filter, options = {}) {
        //options = this.defaultOptions(options);
        if (this.verbose)
            console.debug(TAG$1, "query:", url, filter);
        return this.http.post(url, filter, options).pipe(map((data) => QueryResult.fromData(data)));
    }
    edit(url, value = null, options = {}) {
        if (this.verbose)
            console.debug(TAG$1, "edit:", url, value);
        return (options.method === 'put'
            ? this.http.put(url, value, options)
            : this.http.post(url, value, options)).pipe(map((data) => EditResult.fromData(data)));
    }
    load(url, value = null, options = {}) {
        if (this.verbose)
            console.debug(TAG$1, "load:", url);
        let method = options.method || 'get';
        let obs;
        if (value) {
            obs = this.http[method](url, value, options);
        }
        else {
            obs = this.http[method](url, options);
        }
        return obs;
    }
    download(url, filter = null, options = {}) {
        if (this.verbose)
            console.debug(TAG$1, "download:", url, filter);
        options = options || {};
        options.responseType = options.responseType || 'blob';
        let method = options.method || 'post';
        return this.http[method](url, filter, options);
    }
    checkNotNull(value, message = "Not found") {
        if (value == null)
            throw new Error(message);
        return value;
    }
}

// noinspection DuplicatedCode
/**
 * ts impl of inc.yukawa.chain.base.rest.ctrl.RepoAspectRest
 */
class RepoAspectRest {
    constructor(http, configService, urlKey = 'baseUrl') {
        this.http = http;
        this.configService = configService;
        this.urlKey = urlKey;
    }
    // noinspection JSMethodCanBeStatic
    get verbose() {
        return isDevMode();
    }
    formatServiceUrl(path = '') {
        return this.configService.formatUrl(this.urlKey) + path;
    }
    // -----------------------------------------------------------------------------------------------------------------
    load(key) {
        const queryParamsList = [];
        if (key !== undefined && key !== null) {
            queryParamsList.push({ name: 'key', value: key.toString() });
        }
        let params = new HttpParams();
        for (const queryParam of queryParamsList) {
            params = params.append(queryParam.name, queryParam.value);
        }
        return this.http.get(this.formatServiceUrl('/'), { params });
    }
    count(filter) {
        return this.http.post(this.formatServiceUrl('/count'), filter);
    }
    find(filter) {
        return this.http.post(this.formatServiceUrl('/find'), filter);
    }
    findKeys(filter) {
        return this.http.post(this.formatServiceUrl('/findKeys'), filter);
    }
    query(filter) {
        return this.http.post(this.formatServiceUrl('/query'), filter)
            .pipe(map((data) => QueryResult.fromData(data)));
    }
    // -----------------------------------------------------------------------------------------------------------------
    put(entity) {
        return this.http.post(this.formatServiceUrl('/put'), entity);
    }
    create(entity) {
        return this.http.post(this.formatServiceUrl('/create'), entity);
    }
    update(entity) {
        return this.http.post(this.formatServiceUrl('/update'), entity);
    }
    merge(entity) {
        return this.http.post(this.formatServiceUrl('/merge'), entity);
    }
    edit(entity) {
        throw new Error('edit not implemented');
    }
    // -----------------------------------------------------------------------------------------------------------------
    deleteByKey(key) {
        const queryParamsList = [];
        if (key !== undefined && key !== null) {
            queryParamsList.push({ name: 'key', value: key.toString() });
        }
        let params = new HttpParams();
        for (const queryParam of queryParamsList) {
            params = params.append(queryParam.name, queryParam.value);
        }
        return this.http.delete(this.formatServiceUrl('/'), { params });
    }
    delete(entity) {
        return this.http.post(this.formatServiceUrl('/delete'), entity);
    }
    deleteAll(filter) {
        return this.http.post(this.formatServiceUrl('/deleteAll'), filter);
    }
}

class InterceptedRepoAspectsRest extends RepoAspectRest {
    constructor(http, configService, urlKey) {
        super(http, configService, urlKey);
    }
    load(key) {
        return this.beforeLoad(key)
            .pipe(mergeMap(key => super.load(key)))
            .pipe(mergeMap(entity => this.afterLoad(entity)));
    }
    find(filter) {
        return this.beforeRead(filter)
            .pipe(mergeMap(filter => super.find(filter)))
            .pipe(concatAll())
            .pipe(concatMap(res => this.afterRead(res)))
            .pipe(toArray());
    }
    query(filter) {
        return this.beforeRead(filter)
            .pipe(mergeMap(filter => super.query(filter)))
            .pipe(mergeMap(res => this.afterQuery(res)));
    }
    create(entity) {
        return this.beforeWrite(entity)
            .pipe(mergeMap(entity => super.create(entity)))
            .pipe(mergeMap(entity => this.afterWrite(entity)));
    }
    update(entity) {
        return this.beforeWrite(entity)
            .pipe(mergeMap(entity => super.update(entity)))
            .pipe(mergeMap(entity => this.afterWrite(entity)));
    }
    merge(entity) {
        return this.beforeWrite(entity)
            .pipe(mergeMap(entity => super.merge(entity)))
            .pipe(mergeMap(entity => this.afterWrite(entity)));
    }
    put(entity) {
        return this.beforeWrite(entity)
            .pipe(mergeMap(entity => super.put(entity)))
            .pipe(mergeMap(entity => this.afterWrite(entity)));
    }
    deleteByKey(key) {
        return this.beforeDelete(key)
            .pipe(mergeMap(key => super.deleteByKey(key)))
            .pipe(mergeMap(entity => this.afterDelete(entity)));
    }
    beforeLoad(key) {
        return of(key);
    }
    ;
    beforeRead(filter) {
        return of(filter);
    }
    beforeWrite(entity) {
        return of(entity);
    }
    beforeDelete(key) {
        return of(key);
    }
    afterLoad(entity) {
        return of(entity);
    }
    afterRead(entity) {
        return of(entity);
    }
    afterQuery(result) {
        return of(result);
    }
    afterWrite(entity) {
        return of(entity);
    }
    afterDelete(entity) {
        return of(entity);
    }
}

class CachedRepoAspectRest extends InterceptedRepoAspectsRest {
    constructor(http, configService, urlKey, keyExtractor) {
        super(http, configService, urlKey);
        this.keyExtractor = keyExtractor;
        this.idCache = {};
        this.idCacheRefreshSubjects = {};
        this.queryCacheRefreshSubject = new Subject();
    }
    query(filter) {
        if (!this.queryCache) {
            this.queryCache = this.queryCacheRefreshSubject
                .pipe(delay(1500))
                .pipe(startWith(""))
                .pipe(switchMap(() => super.query(filter)))
                .pipe(shareReplay(1));
        }
        return this.queryCache;
    }
    load(key) {
        if (!this.getRefreshSubject(key)) {
            this.idCacheRefreshSubjects[key.toString()] = new Subject();
        }
        if (!this.getCachedEntityObservable(key)) {
            this.idCache[key.toString()] = this.getRefreshSubject(key)
                .pipe(delay(1500))
                .pipe(startWith(""))
                .pipe(switchMap(() => super.load(key)))
                .pipe(shareReplay(1));
        }
        return this.getCachedEntityObservable(key);
    }
    update(entity) {
        return super.update(entity).pipe(this.refreshOp(entity));
    }
    merge(entity) {
        return super.merge(entity).pipe(this.refreshOp(entity));
    }
    put(entity) {
        return super.put(entity).pipe(this.refreshOp(entity));
    }
    edit(entity) {
        return super.edit(entity).pipe(this.refreshOp(entity));
    }
    refreshOp(entity) {
        return tap(() => {
            this.refreshCaches(entity);
        });
    }
    refresh() {
    }
    refreshCaches(entity) {
        this.refreshEntityCache(this.keyExtractor(entity));
        this.refreshQueryCache();
    }
    getCachedEntityObservable(key) {
        return this.idCache[key.toString()];
    }
    getRefreshSubject(key) {
        return this.idCacheRefreshSubjects[key.toString()];
    }
    refreshEntityCache(key) {
        if (this.getRefreshSubject(key)) {
            this.getRefreshSubject(key).next();
        }
    }
    refreshQueryCache() {
        this.queryCacheRefreshSubject.next();
    }
}

/*
 * Public API Surface of chain-base-angular-client
 */
//import 'chaos-sys-core-1.9.3-SNAPSHOT.min.js';
//export var SysType:  any;

/**
 * Generated bundle index. Do not edit.
 */

export { CachedRepoAspectRest, ConfigFactory, ConfigModule, ConfigService, InterceptedRepoAspectsRest, RepoAspectRest, RestAspect, UrlConfigService, init };
//# sourceMappingURL=yukawa-chain-base-angular-client.js.map
